# Generated by Django 4.1.9 on 2023-06-02 06:15

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbhuman_resources", "0001_initial_squashed_squashed_0015_alter_absencerequest_calendaritem_ptr_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="employeehumanresource",
            options={
                "permissions": [("global_manager", "Is a Global Manager?")],
                "verbose_name": "Employee Human Resource",
                "verbose_name_plural": "Employee Human Resources",
            },
        ),
    ]
