from .absence import (
    AbsenceRequestEmployeeBalanceTitleConfig,
    AbsenceRequestPlannerTitleConfig,
    AbsenceTablePandasTitleConfig,
    AbsenceTypeCountEmployeeTitleConfig,
)
from .employee import EmployeeBalanceTitleConfig, EmployeeTitleConfig
from .kpis import EvaluationGraphTitleConfig, KPIEvaluationPandasTitleConfig
from .review import (
    ReviewAnswerReviewPandasTitleConfig,
    ReviewAnswerReviewTitleConfig,
    ReviewProgressPandasTitleConfig,
    ReviewProgressReviewPandasTitleConfig,
    ReviewQuestionReviewQuestionCategoryTitleConfig,
    ReviewQuestionReviewTitleConfig,
    ReviewReviewGroupTitleConfig,
    ReviewTemplateTitleConfig,
)
